﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Zr.Interface;
using Zr.Interface.Data;

namespace N2InterfaceSample
{
    public partial class MainForm : Form
    {
        Module _Module;

        public MainForm()
        {
            /* contstructor */
            _Module = new Module();
            //_Module = new Module("Test Module", 1);
            //_Module = new Module("Test Module", 1, InterfaceSecurityTypes.Simple);
            //_Module = new Module("Test Module", 1, "127.0.0.1", 19653);
            //_Module = new Module("Test Module", 1, "127.0.0.1", 19653, InterfaceSecurityTypes.Simple);

            /**/
            _Module.ErrorCode += _Module_ErrorCode;

            _Module.Disconnected += _Module_Disconnected;
            _Module.Connected += _Module_Connected;

            /* module authentication received */
            _Module.AuthenticationAck += _Module_AuthenticationAck;
            _Module.AuthenticationNock += _Module_AuthenticationNock;

            /* received value */
            _Module.ReceivedValue += _Module_ReceivedValue;

            InitializeComponent();

            /* module library version */
            Text += $"  ver.1.0.1  (Interface Library ver.{_Module.GetVersion()})";



            /* sample */
            _Module?.SetTag((uint)0, "D00001", InterfaceTagTypes.Digital);
            lstTag.Items.Add(new ListViewItem(new string[] { "D00001", InterfaceTagTypes.Digital.ToString() }));
            _Module?.SetTag((uint)1, "D00002", InterfaceTagTypes.Analog);
            lstTag.Items.Add(new ListViewItem(new string[] { "D00002", InterfaceTagTypes.Analog.ToString() }));
            _Module?.SetTag((uint)2, "D00003", InterfaceTagTypes.String);
            lstTag.Items.Add(new ListViewItem(new string[] { "D00003", InterfaceTagTypes.String.ToString() }));
        }

        private void _Module_ErrorCode(ErrorCodes code, string message)
        {
            _SetDebugLog($"[{code}] {message}");
        }


        /// <summary>
        /// Recevied value from server
        /// </summary>
        /// <param name="name"></param>
        /// <param name="type"></param>
        /// <param name="value"></param>
        private void _Module_ReceivedValue(string name, InterfaceFormatTypes type, object value)
        {
            _SetDebugLog($"Received: Name= {name}, Type= {type}, Value= {value}");

            /* casting (InterfaceFormatTypes) */
            switch (type)
            {
                case InterfaceFormatTypes.Float:
                    var floatValue = (float)value;
                    break;
                case InterfaceFormatTypes.Double:
                    var doubleValue = (double)value;
                    break;
                case InterfaceFormatTypes.Boolean:
                    var boolValue = (bool)value;
                    break;

                case InterfaceFormatTypes.String:
                    var stringValue = (string)value;
                    break;

                case InterfaceFormatTypes.Json:
                    /* json format(custom) */
                    var jsonValue = (string)value;
                    break;
            }
        }

        /// <summary>
        /// Received authentication
        /// </summary>
        /// <param name="key">sysmte(module) number</param>
        private void _Module_AuthenticationAck(byte key)
        {
            lblAuthenticationState.Text = $"Authentication Num= {key}";

            _SetDebugLog($"Authentication Num= {key}");

            /* sending synchronize data */
            //_Module.SynchronizeOnName(new List<SyncItem>() {
            //    new NameSyncItem("Tag00001", InterfaceFormatTypes.Double, BitConverter.GetBytes(0.0) ) });
            //_Module.SynchronizeOnName(new List<SyncItem>() {
            //    new IndexSyncItem(0, InterfaceFormatTypes.Double, BitConverter.GetBytes(0.0)) });
        }

        private void _Module_AuthenticationNock(AliveResultTypes resultType)
        {
            lblAuthenticationState.Text = $"Authentication Fail= {resultType}";

            _SetDebugLog($"Authentication Fail = {resultType}");
        }

        private void _Module_Connected()
        {
            lblConnectionState.Text = "Connected";

            _SetDebugLog("Connected");
        }

        private void _Module_Disconnected(_ClientBase obj)
        {
            lblConnectionState.Text = "Disconnected";

            _SetDebugLog("Disconnected");

            lblAuthenticationState.Text = $"No Authentication.";
        }




        private void _SetDebugLog(string message)
        {
            txtDebug.AppendText($"{DateTime.Now.ToString("HH:mm:ss")} {message}{Environment.NewLine}");
        }




        /// <summary>
        /// Start server connection
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnStart_Click(object sender, EventArgs e)
        {
            btnStart.Enabled = false;

            try
            {
                string moduleName = txtModuleName.Text;
                byte systemNumber = byte.Parse(txtSystemNumber.Text);

                string authenticationKey = txtAuthenticationKey.Text;

                string ip = txtSrvIp.Text;
                int port = int.Parse(txtSrvPort.Text);

                /* initialize */
                _Module.Initialize(moduleName, ip, port);

                /* set system number - (0x00 < system number < 0xA0) */
                _Module.SetSystemNumber(systemNumber);

                /* set security type */
                if (rbtnSecNo.Checked)
                {
                    /* Not use */
                    _Module.SetSecurityType(InterfaceSecurityTypes.None);
                }
                else if (rbtnSecSimple.Checked)
                {
                    /* simple security */
                    _Module.SetSecurityType(InterfaceSecurityTypes.Simple);
                }
                else if (rbtnSecSymm.Checked)
                {
                    /* Set Authentication key
                     *  - Length == 8 */
                    if (String.IsNullOrEmpty(authenticationKey))
                        throw new Exception("Authentication key is null");
                    if (authenticationKey?.Length != 8)
                        throw new Exception("The authentication key is less than 8 characters.");

                    /**/
                    _Module.SetSecurityType(InterfaceSecurityTypes.SymmetricKey);

                    _Module.SetAuthenticationKey(authenticationKey);
                }
                else if (rbtnSecAsymm.Checked)
                {
                    /* Set Authentication key 
                     *  - Length == 8 */
                    if (String.IsNullOrEmpty(authenticationKey))
                        throw new Exception("Authentication key is null");
                    if (authenticationKey?.Length != 8)
                        throw new Exception("The authentication key is less than 8 characters.");

                    /**/
                    _Module.SetSecurityType(InterfaceSecurityTypes.AsymmetricKey);

                    _Module.SetAuthenticationKey(authenticationKey);
                }
                else
                    throw new NotSupportedException("Security Type");

                /* Start - retry connection delay */
                _Module.Start(1000);
            }
            catch (Exception err)
            {
                MessageBox.Show(this, err.Message);
            }
            finally
            {
                btnStop.Enabled = true;
            }
        }

        /// <summary>
        /// Stop server connection
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnStop_Click(object sender, EventArgs e)
        {
            btnStop.Enabled = false;

            try
            {
                /* stop connection */
                _Module?.Stop();
            }
            catch (Exception err)
            {
                MessageBox.Show(this, err.Message);
            }
            finally
            {
                btnStart.Enabled = true;
            }
        }

        /// <summary>
        /// Add tag item
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnAddTag_Click(object sender, EventArgs e)
        {
            try
            {
                string name = txtTagName.Text;

                if (!String.IsNullOrEmpty(name))
                {
                    /* enum type */
                    InterfaceTagTypes type = InterfaceTagTypes.Unknown;
                    /* string type */
                    //string type = String.Empty;

                    if (rbtnTypeDigital.Checked)
                    {
                        type = InterfaceTagTypes.Digital;
                        //type = "digital" or "d" or "0";
                    }
                    else if (rbtnTypeAnalog.Checked)
                    {
                        type = InterfaceTagTypes.Analog;
                        //type = "analog" or "a" or "1";
                    }
                    else if (rbtnTypeString.Checked)
                    {
                        type = InterfaceTagTypes.String;
                        //type = "string" or "s" or "3";
                    }
                    else
                        throw new NotSupportedException("Type");

                    /* add(set) tag - (enum type or string type) 
                     *  - Tag Index, (0 ~)
                     *  - Tag Name, (Unique name)
                     *  - Tag Type, (Digital, Analog, String)
                     */
                    var error = _Module?.SetTag((uint)lstTag.Items.Count, name, type);

                    switch (error)
                    {
                        case ErrorCodes.Success:
                            lstTag.Items.Add(new ListViewItem(new string[] { name, type.ToString() }));
                            break;

                        default:
                            break;
                    }

                    _SetDebugLog($"Set tag item result = {error}");
                }
                else
                    throw new Exception("Tag name is empty");
            }
            catch (Exception err)
            {
                _SetDebugLog($"[E] add tag items - {err.Message}");
            }
        }

        /// <summary>
        /// Release tag item
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnDelTag_Click(object sender, EventArgs e)
        {
            try
            {
                var items = lstTag.SelectedItems;

                if (items?.Count > 0)
                {
                    string tagName = items[0].SubItems[0].Text;

                    if (!String.IsNullOrEmpty(tagName))
                    {
                        /* release tag */
                        var error = _Module?.ReleaseTag(tagName);

                        switch (error)
                        {
                            case ErrorCodes.Success:
                                lstTag.Items.Remove(items[0]);
                                break;

                            default:
                                break;
                        }

                        _SetDebugLog($"Release tag item result = {error}");
                    }
                    else
                        throw new Exception("Tag name is null");
                }
                else
                    throw new Exception("Selected is null");
            }
            catch (Exception err)
            {
                _SetDebugLog($"[E] delete tag items - {err.Message}");
            }
        }

        /// <summary>
        /// Clear tag item
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnClrTag_Click(object sender, EventArgs e)
        {
            try
            {
                /* clear tag */
                var error = _Module?.ClearTags();

                switch (error)
                {
                    case ErrorCodes.Success:
                        lstTag.Items.Clear();
                        break;

                    default:
                        break;
                }

                _SetDebugLog($"Clear tag item result = {error}");
            }
            catch (Exception err)
            {
                _SetDebugLog($"[E] clear tag items - {err.Message}");
            }
        }


        /// <summary>
        /// Send to server 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSend_Click(object sender, EventArgs e)
        {
            try
            {
                string tagName = txtSendTagName.Text;
                string tagType = lblSendTagType.Text;

                if (String.IsNullOrEmpty(tagName))
                    throw new Exception("Tag name is null");

                ErrorCodes error = ErrorCodes.NotUse;

                switch (tagType)
                {
                    case "Digital":
                        /* send change value(boolean) */
                        error = _Module.Change(tagName, rbtnTrueValue.Checked);
                        break;

                    case "Analog":
                        /* send change value(double) */
                        error = _Module.Change(tagName, double.Parse(txtSendValue.Text));
                        break;

                    case "String":
                    default:
                        /* send change value(string) */
                        error = _Module.Change(tagName, txtSendValue.Text);
                        break;
                }

                switch (error)
                {
                    case ErrorCodes.Success:
                        break;

                    default:
                        break;
                }

                _SetDebugLog($"Send value, Name: {tagName}, Type:{tagType}, Result: {error}");
            }
            catch (Exception err)
            {
                _SetDebugLog($"[E] send - {err.Message}");
            }
        }


        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void lstTag_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                var items = lstTag.SelectedItems;

                if (items?.Count > 0)
                {
                    txtSendTagName.Text = items[0].SubItems[0].Text;
                    lblSendTagType.Text = items[0].SubItems[1].Text;
                }
            }
            catch (Exception err)
            {
                _SetDebugLog($"[E] selection change (list view) - {err.Message}");
            }
        }

        private void MainForm_Load(object sender, EventArgs e)
        {

        }
    }
}
